
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsiDecl.h"

namespace gsi
{

template <> struct ClassTag<gsi::ClassBase> {
  typedef CantCopySemanticsTag copy_semantics;
  typedef CantDefaultCreateSemanticsTag create_semantics;
};

template <> struct ClassTag<gsi::MethodBase> {
  typedef CantCopySemanticsTag copy_semantics;
  typedef CantDefaultCreateSemanticsTag create_semantics;
};

// ---------------------------------------------------------------
//  Access to the internal informations of GSI

static int t_void () { return T_void; }
static int t_bool () { return T_bool; }
static int t_int () { return T_int; }
static int t_uint () { return T_uint; }
static int t_long () { return T_long; }
static int t_ulong () { return T_ulong; }
static int t_longlong () { return T_longlong; }
static int t_double () { return T_double; }
static int t_string_ccptr () { return T_string_ccptr; }
static int t_string () { return T_string; }
static int t_var () { return T_var; }
static int t_object_ref () { return T_object_ref; }
static int t_object_cref () { return T_object_cref; }
static int t_object_new () { return T_object_new; }
static int t_object () { return T_object; }

static int type (const ArgType *t) 
{
  return t->type;
}

static bool vector (const ArgType *t) 
{
  return t->vector;
}

static bool ref (const ArgType *t) 
{
  return t->ref;
}

static bool iter (const ArgType *t) 
{
  return t->iter;
}

static const ClassBase *cls (const ArgType *t) 
{
  return t->cls;
}

template <>
Class<ArgType> Class<ArgType>::decl ("ArgType", 
  gsi::method ("t_void", &t_void) +
  gsi::method ("t_bool", &t_bool) +
  gsi::method ("t_int", &t_int) +
  gsi::method ("t_uint", &t_uint) +
  gsi::method ("t_long", &t_long) +
  gsi::method ("t_ulong", &t_ulong) +
  gsi::method ("t_longlong", &t_longlong) +
  gsi::method ("t_double", &t_double) +
  gsi::method ("t_string_ccptr", &t_string_ccptr) +
  gsi::method ("t_string", &t_string) +
  gsi::method ("t_var", &t_var) +
  gsi::method ("t_object_ref", &t_object_ref) +
  gsi::method ("t_object_cref", &t_object_cref) +
  gsi::method ("t_object_new", &t_object_new) +
  gsi::method ("t_object", &t_object) +
  gsi::method_ext ("type", &type,
    "@brief Return the basic type (see t_.. constants)\n"
  ) +
  gsi::method_ext ("is_vector?", &vector,
    "@brief True, if the type is a vector of the basic type\n"
  ) +
  gsi::method_ext ("is_ref?", &ref,
    "@brief True, if the type is a reference to the given type\n"
  ) +
  gsi::method_ext ("is_iter?", &iter,
    "@brief (Return value only) True, if the return value is an iterator rendering the given type\n"
  ) +
  gsi::method_ext ("cls", &cls,
    "@brief Specifies the class for t_object.. types\n"
  ) +
  gsi::method ("to_s", &ArgType::to_string,
    "@brief Convert to a string\n"
  ) +
  gsi::method ("==", &ArgType::operator==,
    "@brief Equality of two types\n"
  ) +
  gsi::method ("!=", &ArgType::operator!=,
    "@brief Inequality of two types\n"
  ),
  "@brief The description of a type (argument or return value)"
);

template <>
Class<MethodBase> Class<MethodBase>::decl ("Method", 
  gsi::iterator ("each_argument", &MethodBase::begin_arguments, &MethodBase::end_arguments,
    "@brief Iterate over all arguments of this method\n"
  ) +
  gsi::method ("ret_type", &MethodBase::ret_type,
    "@brief The return type of this method\n"
  ) +
  gsi::method ("is_const?", &MethodBase::is_const,
    "@brief True, if this method does not alter the object\n"
  ) +
  gsi::method ("is_static?", &MethodBase::is_static,
    "@brief True, if this method is static (a class method)\n"
  ) +
  gsi::method ("is_event?", &MethodBase::is_event,
    "@brief True, if this method is an event\n"
  ) +
  gsi::method ("name", &MethodBase::name,
    "@brief The name of the class\n"
  ) +
  gsi::method ("doc", &MethodBase::doc,
    "@brief The documentation string for this method\n"
  ),
  "@brief The interface to a method declaration"
);

template <>
Class<ClassBase> Class<ClassBase>::decl ("Class", 
  gsi::iterator ("each_class", &ClassBase::begin_classes, &ClassBase::end_classes,
    "@brief Iterate over all classes\n"
  ) +
  gsi::iterator ("each_method", &ClassBase::begin_methods, &ClassBase::end_methods,
    "@brief Iterate over all methods of this class\n"
  ) +
  gsi::method ("name", &ClassBase::name,
    "@brief The name of the class\n"
  ) +
  gsi::method ("can_copy", &ClassBase::can_copy,
    "@brief True if the class offers assignment\n"
  ) +
  gsi::method ("doc", &ClassBase::doc,
    "@brief The documentation string for this class\n"
  ),
  "@brief The interface to the declarations of classes and methods"
);

}


