
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsiDecl.h"
#include "tlObserver.h"

namespace gsi
{

//  The Observer stub to allow reimplementation of the signal_observer methods
class ObserverStub
  : tl::Observer
{
public:
  virtual void signal_observed ()
  {
    if (gsi::can_issue (signal_observed_cb)) {
      gsi::issue (signal_observed_cb, &tl::Observer::signal_observed);
    }
  }

  virtual void signal_observed_int (int n)
  {
    if (gsi::can_issue (signal_observed_int_cb)) {
      gsi::issue (signal_observed_int_cb, &tl::Observer::signal_observed_int, n);
    }
  }

  gsi::Callback signal_observed_cb;
  gsi::Callback signal_observed_int_cb;
};

template <>
Class<tl::Observer> Class<tl::Observer>::decl ("ObserverBase",
  "@brief The 'Observer' base class\n"
  "\n"
  "This class acts as the base class for observer objects. The implementation must be "
  "provided through \\Observer."
);

template <>
Class<ObserverStub> Class<ObserverStub>::decl (Class<tl::Observer>::decl, "Observer", 
  callback ("signal", &ObserverStub::signal_observed, &ObserverStub::signal_observed_cb,
    "@brief This method is called when the event without value is issued"
  ) +
  callback ("signal_int", &ObserverStub::signal_observed_int, &ObserverStub::signal_observed_int_cb,
    "@brief This method is called when an event associated with an integer is issued\n"
    "@args value"
  ),
  "@brief This class implements an event handler for use with 'observer' interfaces\n"
  "\n"
  "Some classes provide callbacks by attaching 'observer' objects to certain events. "
  "Specific actions can be implemented by reimplementing the 'signal...' methods of this class. "
);

}

