
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layApplication.h"
#include "layExceptions.h"
#include "layFileDialog.h"
#include "tlInternational.h"
#include "tlException.h"
#include "tlLog.h"
#include "rba.h"

#include <iostream>
#include <memory>

#ifdef _MSC_VER // for VC++ 

//  for VC++ provide a wrapper for main.
#include <Windows.h>

int main (int argc, char *argv []);

int WINAPI 
wWinMain(HINSTANCE hInstance, HINSTANCE prevInstance, LPWSTR lpCmdLine, int nShowCmd)
{
  int argCount = 0;
  LPWSTR *szArgList = CommandLineToArgvW(GetCommandLine(), &argCount);

  //  fail safe behaviour
  if (!szArgList) {
	  MessageBox(NULL, L"Unable to parse command line", L"Error", MB_OK);
	  return 10;
  }

  char **argv = new char *[argCount];
  for (int i = 0; i < argCount; i++) {
	  QString a;
	  for (WCHAR *wc = szArgList [i]; *wc; ++wc) {
		  a += QChar (*wc);
	  }
	  QByteArray aa = a.toAscii ();
	  argv [i] = new char [aa.size () + 1];
	  strcpy (argv [i], aa.constData ());
  }

  int ret = main (argCount, argv);

  for (int i = 0; i < argCount; i++) {
	  delete[] argv [i];
  }
  delete[] argv;

  LocalFree(szArgList);
  return ret;
}
#endif

int 
main (int argc, char *argv [])
{
#if defined(Q_WS_X11) || defined(Q_WS_MAC)
  //  HACK: this is a crude way to switch the file dialog's to UTF8 in order to handle 
  //  UTF8 file systems properly. A real solution would be to make all strings involved
  //  UniCode or UTF8 for std::string().
  const char *lang = getenv ("LANG");
  if (lang && strstr (lang, "8") && (strstr (lang, "UTF") || strstr (lang, "utf"))) {
    lay::FileDialog::set_utf8 (true);
  }

  //  Force number format to "C". This is done by setting the environment variable 
  //  because otherwise QApplication tends to reset our setting.
  //  LC_NUMERIC=C is required to force "C" locale for numbers, resetting LC_ALL is 
  //  required to remove any global settings form the environment (in that case, LANG will
  //  determine the locale).
  putenv ((char *) "LC_NUMERIC=C");
  putenv ((char *) "LC_ALL=");

  //  HINT: the following statement overrides the LANG locale provided by the system.
  //  - The reason is, that Qt employs a function called _XInitOM that seems not to be 
  //    capable of handling "UTF8" LANG types. This makes QTextEdit widgets with HTML 
  //    content SIGSEGV when text is marked.
  //    putenv ((char *) "LANG=C");
  //  This setting cannot be used in the general case, because this will inhibit KDE's 
  //  file dialog to correctly handle UTF-8 paths.
#endif

  int result = 0;

  try {

    //  see comment there why this call is here.
    rba::RubyInterpreter::initialize ();

#if KLAYOUT_VIEWER_ONLY
    std::auto_ptr<char *> av (new char *[argc + 1]);
    for (int i = 0; i < argc; ++i) {
      av.get () [i] = argv [i];
    }
    char ne_param[] = "-ne";
    argv = av.get ();
    argv [argc] = ne_param;
    argc += 1;
#endif

    lay::Application app (argc, argv);

    if (app.has_gui ()) {

      BEGIN_PROTECTED_CLEANUP

      result = app.run ();

      END_PROTECTED_CLEANUP {
        result = 1;
      }

    } else {
      result = app.run ();
    }

  } catch (std::exception &ex) {
    tl::error << ex.what ();
    result = 1;
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    result = 1;
  } catch (...) {
    tl::error << tl::translate ("unspecific error");
    result = 1;
  }

  return result;

}


