
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layTechSetupDialog.h"
#include "layTechnology.h"
#include "layExceptions.h"

#include <stdio.h>

namespace lay
{

// ----------------------------------------------------------------
//  TechSetupDialog implementation

TechSetupDialog::TechSetupDialog (QWidget *parent)
  : QDialog (parent, "tech_setup_dialog")
{
  Ui::TechSetupDialog::setupUi (this);
}

// ----------------------------------------------------------------
//  TechComponentSetupDialog implementation

TechComponentSetupDialog::TechComponentSetupDialog (QWidget *parent, Technology *tech, const std::string &component_name)
  : QDialog (parent, "tech_component_setup_dialog"),
    mp_editor (0)
{
  Ui::TechComponentSetupDialog::setupUi (this);

  TechnologyComponent *component = tech->component_by_name (component_name);
  if (component) {

    mp_editor = component->create_editor (content_frame);
    if (mp_editor) {

      QVBoxLayout *layout = new QVBoxLayout (content_frame);
      layout->addWidget (mp_editor);
      layout->setMargin (0);
      content_frame->setLayout (layout);

      mp_editor->set_technology (tech, component);
      mp_editor->setup ();

    }
  }
}

void 
TechComponentSetupDialog::accept ()
{
BEGIN_PROTECTED
  if (mp_editor) {
    mp_editor->commit ();
  }

  QDialog::accept ();
END_PROTECTED
}

}

