
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef _HDR_rba
#define _HDR_rba

#include "gsi.h"

namespace rba
{

/**
 *  @brief A class encapsulating a ruby exception
 */
class KLAYOUT_DLL RubyError
  : public tl::Exception 
{
public:
  RubyError (const char *msg, const char *cls, const std::vector <std::string> &backtrace)
    : tl::Exception (msg), m_line (-1), m_cls (cls), m_backtrace (backtrace)
  { }

  RubyError (const char *msg, const char *sourcefile, int line, const char *cls, const std::vector <std::string> &backtrace)
    : tl::Exception (msg), m_sourcefile (sourcefile), m_line (line), m_cls (cls), m_backtrace (backtrace)
  { }

  const std::string &sourcefile () const               { return m_sourcefile; }
  int line () const                                    { return m_line; }
  const std::string &cls () const                      { return m_cls; }
  const std::vector<std::string> &backtrace () const   { return m_backtrace; }

  virtual std::string msg () const;

  std::string basic_msg () const
  {
    return tl::Exception::msg ();
  }

private:
  std::string m_sourcefile;
  int m_line;
  std::string m_cls;
  std::vector<std::string> m_backtrace;
};

/**
 *  @brief The console interface for output
 *
 *  Using this interface, the interpreter can output text on stdout or stderr streams.
 *  This interface is implemented by ConsoleImpl.
 */
class Console
{
public:
  enum output_stream { OS_none, OS_stdout, OS_stderr, OS_echo };

  Console () { }
  virtual ~Console () { }

  virtual void write_str (const char *text, output_stream os) = 0;
};

/**
 *  @brief The ruby interpreter wrapper class
 */
class KLAYOUT_DLL RubyInterpreter
{
public:
  RubyInterpreter ();
  ~RubyInterpreter ();

  void require (const char *filename);
  void load_file (const char *filename);
  void eval_string (const char *string);
  void eval_string_and_print (const char *string);
  void define_variable (const std::string &name, const std::string &value);
  bool ruby_available () const;
  void install_console (Console *console);

  /**
   *  @brief Provide a first (basic) initialization
   */
  static void initialize ();
};

}

#endif


