
/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "utHead.h"
#include "rba.h"

#include <stdio.h>
#include <Windows.h>

void debug_print (const std::string &s) 
{
  OutputDebugStringA (s.c_str ());
  OutputDebugStringA ("\n");
}

namespace ut 
{

ut::Registrar *ut::Registrar::ms_instance = 0;

static bool _verbose_flag = false;

bool verbose ()
{
  return _verbose_flag;
}

} // namespace ut

rba::RubyInterpreter *ruby_interp_instance = 0;

int 
main (int argc, const char *argv[]) 
{
  //  With Ruby, "fclose" is redirected to the Ruby-DLL. Because of that, Ruby needs to be initialized before fclose can be used ...
  rba::RubyInterpreter::initialize ();
  rba::RubyInterpreter rbinterp;
  ruby_interp_instance = &rbinterp;

  ut::_verbose_flag = false;
  int failed_ne, failed_e, failed;

  debug_print ("------------------------------------------------------------------------------");
  debug_print ("Running tests in non-editable mode ...");
  db::set_editable_mode (false);

  failed_ne = ut::Registrar::do_tests ();
  if (failed_ne > 0) {
    debug_print (std::string ("Summary: ") + tl::to_string (failed_ne) + " test(s) failed");
  } else {
    debug_print ("All tests passed in non-editable mode.");
  }

  debug_print ("------------------------------------------------------------------------------");
  debug_print ("Running tests in editable mode ...");
  db::set_editable_mode (true);

  failed_e = ut::Registrar::do_tests ();
  if (failed_e > 0) {
    debug_print (std::string ("Summary: ") + tl::to_string (failed_e) + " test(s) failed");
  } else {
    debug_print ("All tests passed in editable mode.");
  }

  debug_print ("------------------------------------------------------------------------------");
  failed = failed_e + failed_ne;
  if (failed > 0) {
    debug_print (std::string ("Grand summary: ") + tl::to_string (failed) + " test(s) failed");
  } else {
    debug_print ("All tests passed.");
  }

  return failed;
}

