
use_minitest = (RUBY_VERSION =~ /^1\.9/)

if use_minitest
  require "minitest/unit"
  test_base = MiniTest::Unit::TestCase
else
  require "test/unit"
  test_base = Test::Unit::TestCase
end

class Test_imgObject < test_base

  def test_1

    dm = RBA::ImageDataMapping.new 
    dm.gamma = 0.5
    assert_equal(dm.gamma, 0.5)
    dm.brightness = 0.25
    assert_equal(dm.brightness, 0.25)
    dm.contrast = -0.125
    assert_equal(dm.contrast, -0.125)
    dm.red_gain = 1.25
    assert_equal(dm.red_gain, 1.25)
    dm.green_gain = 1.125
    assert_equal(dm.green_gain, 1.125)
    dm.blue_gain = 0.125
    assert_equal(dm.blue_gain, 0.125)

    assert_equal(dm.num_colormap_entries, 2)
    assert_equal(dm.colormap_color(0) & 0xffffff, 0x000000)
    assert_equal(dm.colormap_color(1) & 0xffffff, 0xffffff)

    dm.clear_colormap
    assert_equal(dm.num_colormap_entries, 0)
    dm.add_colormap_entry(0, 0x123456);
    dm.add_colormap_entry(0.5, 0xff0000);
    dm.add_colormap_entry(1.0, 0x00ff00);
    assert_equal(dm.num_colormap_entries, 3)
    assert_equal(dm.colormap_color(0) & 0xffffff, 0x123456)
    assert_equal(dm.colormap_value(0), 0.0)
    assert_equal(dm.colormap_color(1) & 0xffffff, 0xff0000)
    assert_equal(dm.colormap_value(1), 0.5)
    assert_equal(dm.colormap_color(2) & 0xffffff, 0x00ff00)
    assert_equal(dm.colormap_value(2), 1.0)

  end

  def test_2

    image = RBA::Image.new 
    assert_equal(image.to_s, "empty:")
    assert_equal(image.is_empty?, true)

    image = RBA::Image.new(2, 3, [])
    assert_equal(image.to_s, "mono:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0;0;0;0;0;]")

    image = RBA::Image.new(2, 3, [], [], [])
    assert_equal(image.to_s, "color:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,0,0;0,0,0;0,0,0;0,0,0;0,0,0;0,0,0;]")

    data = [0.0, 0.5, 1.5, 2.5, 10, 20]
    image = RBA::Image.new(2, 3, data)
    assert_equal(image.to_s, "mono:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0.5;1.5;2.5;10;20;]")

    data2 = [1.0, 1.5, 2.5, 3.5]
    image = RBA::Image.new(2, 3, data, data2, [])
    assert_equal(image.to_s, "color:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")

    t = RBA::DCplxTrans.new(2.5, 90, false, RBA::DPoint.new(1, 5))

    image = RBA::Image.new(2, 3, t, [])
    assert_equal(image.to_s, "mono:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0;0;0;0;0;]")

    image = RBA::Image.new(2, 3, t, [], [], [])
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,0,0;0,0,0;0,0,0;0,0,0;0,0,0;0,0,0;]")

    data = [0.0, 0.5, 1.5, 2.5, 10, 20]
    data2 = [1.0, 1.5, 2.5, 3.5]

    image = RBA::Image.new(2, 3, t, data)
    assert_equal(image.to_s, "mono:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0.5;1.5;2.5;10;20;]")
    assert_equal(image.is_empty?, false)
    assert_equal(image.is_color?, false)

    image.set_pixel(1, 2, 100)
    assert_equal(image.to_s, "mono:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0.5;1.5;2.5;10;100;]")
    assert_equal(image.get_pixel(1, 2), 100)

    image.set_data(2, 3, data, data2, [])
    copy1 = image.dup
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")

    image = RBA::Image.new(2, 3, t, data, data2, [])
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")
    assert_equal(image.is_empty?, false)
    assert_equal(image.is_color?, true)

    image.set_pixel(1, 2, 100, 200, 300)
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;100,200,300;]")
    assert_equal(image.get_pixel(1, 2, 0), 100)
    assert_equal(image.get_pixel(1, 2, 1), 200)
    assert_equal(image.get_pixel(1, 2, 2), 300)

    image.set_data(2, 3, data)
    assert_equal(image.to_s, "mono:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0.5;1.5;2.5;10;20;]")

    assert_equal(image.width, 2);
    assert_equal(image.height, 3);
    assert_equal(image.box.to_s, "(-6.5,5;1,10)")
    image.trans = RBA::DCplxTrans.new
    assert_equal(image.box.to_s, "(0,0;2,3)")
    image.pixel_width = 4
    image.pixel_height = 5
    assert_equal(image.box.to_s, "(0,0;8,15)")
    assert_equal(image.pixel_width, 4)
    assert_equal(image.pixel_height, 5)

    image = image.transformed_cplx(RBA::DCplxTrans.new(1, 0, false, RBA::DPoint.new(1, 2))) 
    assert_equal(image.box.to_s, "(1,2;9,17)")
    assert_equal(image.trans.to_s, "r0 *1 1,2")

    assert_equal(image.filename, "")

    if false 
      image = RBA::Image.new("/home/matthias/a.png")
      assert_equal(image.trans.to_s, "r0 *1 0,0")
      assert_equal(image.width, 728)
      assert_equal(image.height, 762)
      assert_equal(image.filename, "/home/matthias/a.png")
      
      image = RBA::Image.new("/home/matthias/a.png", t)
      assert_equal(image.trans.to_s, "r90 *2.5 1,5")
      assert_equal(image.width, 728)
      assert_equal(image.height, 762)
    end
    
    image.min_value = -12.5
    assert_equal(image.min_value, -12.5)
    image.max_value = 12.5
    assert_equal(image.max_value, 12.5)
    assert_equal(image.id != 0, true)

    image = copy1
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")

    dm = image.data_mapping.dup
    dm.brightness=0.5
    image.data_mapping = dm
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0.5;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")

    assert_equal(image.is_visible?, true)
    image.visible = false
    assert_equal(image.is_visible?, false)
    assert_equal(image.to_s, "color:trans=r90 *2.5 1,5;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=false;brightness=0.5;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0,1,0;0.5,1.5,0;1.5,2.5,0;2.5,3.5,0;10,0,0;20,0,0;]")

  end

  def test_3

    mw = RBA::Application.instance.main_window
    mw.create_layout(0)
    view = mw.current_view

    image = RBA::Image.new(2, 3, [])
    assert_equal(image.to_s, "mono:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0;0;0;0;0;]")

    count = 0
    view.each_image { |i| count += 1 }
    assert_equal(count, 0)

    view.insert_image(image)

    s = ""
    count = 0
    view.each_image { |i| count += 1; s += i.to_s }
    assert_equal(count, 1)
    assert_equal(s, "mono:trans=r0 *1 0,0;pixel_width=1;pixel_height=1;min_value=0;max_value=1;is_visible=true;brightness=0;contrast=0;gamma=1;red_gain=1;green_gain=1;blue_gain=1;color_mapping=[0,'#000000';1,'#ffffff';];width=2;height=3;data=[0;0;0;0;0;0;]")

    view.clear_images
    count = 0
    view.each_image { |i| count += 1 }
    assert_equal(count, 0)

    image = RBA::Image.new(2, 3, [])
    image2 = RBA::Image.new(20, 30, [])

    view.insert_image(image)
    view.insert_image(image2)

    wsum = 0
    hsum = 0
    view.each_image { |i| wsum += i.width; hsum += i.height }
    assert_equal(wsum, 22)
    assert_equal(hsum, 33)

    view.erase_image(image.id)

    wsum = 0
    hsum = 0
    view.each_image { |i| wsum += i.width; hsum += i.height }
    assert_equal(wsum, 20)
    assert_equal(hsum, 30)

    new_image2 = RBA::Image.new(12, 13, [])
    view.replace_image(image2.id, new_image2)

    wsum = 0
    hsum = 0
    view.each_image { |i| wsum += i.width; hsum += i.height }
    assert_equal(wsum, 12)
    assert_equal(hsum, 13)

    visible = 0
    total = 0
    view.each_image { |i| visible += 1 if i.is_visible?; total += 1 }
    assert_equal(visible, 1)
    assert_equal(total, 1)

    view.show_image(new_image2.id, false)

    visible = 0
    total = 0
    view.each_image { |i| visible += 1 if i.is_visible?; total += 1 }
    assert_equal(visible, 0)
    assert_equal(total, 1)

  end

end

# In the test environment, we cannot make sure that we destroy the ruby interpreter before the RBA
# environment is shut down. Therefore we must release all RBA objects by explicitly calling the GC
# and start the test suite manually.

if !use_minitest
  # Ruby 1.8.7 with test/unit:
  require 'test/unit/ui/console/testrunner'
  res = Test::Unit::UI::Console::TestRunner.run(Test_imgObject) 
  res = res.error_count + res.failure_count
else
  # Ruby 1.9.1 with minitest/unit:
  # Make sure that _test_01 is run as the first one - to have a zero instance count of "A"
  res = MiniTest::Unit.new.run(["-n", "/^test_FIRST$/"])
  res ||= MiniTest::Unit.new.run(["-n", "/^test_\\d/"])
end

GC.start

exit 1 unless !res || res == 0

