
use_minitest = (RUBY_VERSION =~ /^1\.9/)

if use_minitest
  require "minitest/unit"
  test_base = MiniTest::Unit::TestCase
else
  require "test/unit"
  test_base = Test::Unit::TestCase
end

class TestImpl < test_base

  def test_1

    app = RBA::Application.instance
    mw = app.main_window
    mw.create_layout(0)
    cv = mw.current_view

    m = RBA::Marker.new(cv)
    m.set(RBA::DBox.new(1, 2, 3, 4))
    m.set(RBA::DText.new("T", RBA::DTrans.new(4)))
    m.set(RBA::DPolygon.new([RBA::DPoint.new(1, 2), RBA::DPoint.new(3, 4), RBA::DPoint.new(5, 6)]))
    m.set(RBA::DPath.new([RBA::DPoint.new(1, 2), RBA::DPoint.new(3, 4), RBA::DPoint.new(5, 6)], 0.5))
    m.set_box(RBA::DBox.new(1, 2, 3, 4))
    m.set_text(RBA::DText.new("T", RBA::DTrans.new(4)))
    m.set_polygon(RBA::DPolygon.new([RBA::DPoint.new(1, 2), RBA::DPoint.new(3, 4), RBA::DPoint.new(5, 7)]))
    m.set_path(RBA::DPath.new([RBA::DPoint.new(1, 2), RBA::DPoint.new(3, 4), RBA::DPoint.new(5, 7)], 0.5))

    m.color = 0xff812345
    assert_equal(m.color, 0xff812345)
    assert_equal(m.has_color?, true)
    m.reset_color
    assert_equal(m.has_color?, false)
    m.color = 0xff4123c5

    m.frame_color = 0xffa23456
    assert_equal(m.frame_color, 0xffa23456)
    assert_equal(m.has_frame_color?, true)
    m.reset_frame_color
    assert_equal(m.has_frame_color?, false)
    m.frame_color = 0xffa23456

    m.line_width = 2
    assert_equal(m.line_width, 2)

    m.vertex_size = 3
    assert_equal(m.vertex_size, 3)

    m.halo = 1
    assert_equal(m.halo, 1)

    m.dither_pattern = 15
    assert_equal(m.dither_pattern, 15)

    # Keep the marker alive after GC.start:
    # $marker = m

  end

end

# In the test environment, we cannot make sure that we destroy the ruby interpreter before the RBA
# environment is shut down. Therefore we must release all RBA objects by explicitly calling the GC
# and start the test suite manually.

if !use_minitest
  # Ruby 1.8.7 with test/unit:
  require 'test/unit/ui/console/testrunner'
  res = Test::Unit::UI::Console::TestRunner.run(TestImpl) 
  res = res.error_count + res.failure_count
else
  # Ruby 1.9.1 with minitest/unit:
  # Make sure that _test_01 is run as the first one - to have a zero instance count of "A"
  res = MiniTest::Unit.new.run(["-n", "/^test_FIRST$/"])
  res ||= MiniTest::Unit.new.run(["-n", "/^test_\\d/"])
end

GC.start

exit 1 unless !res || res == 0

